<?php
/**
 * Created by JetBrains PhpStorm.
 * User: gustavoavila
 * Date: 23/10/12
 * Time: 15:29
 * To change this template use File | Settings | File Templates.
 */
class ger80908_02Logic extends FctLogic
{

    static $_storedInSession = array("IDUTUFORIG","IDUTUFDEST","IDUTFAIXVLR","IDUTREDUBC","ID1UTREDUBC","IDUTPARCDED","IDTPIMPOSFIS");

    public function __construct($useSession = true)
    {
        parent::__construct(new DadosfiscregraEntity());
        if($useSession) {
            parent::storedInSession(self::$_storedInSession, ger80900_01::WINDOW_NAME);
        }
    }

    public function validaInsert($cdregrafisc, $cdimposto, $nrseqdadosfisc)
    {
        $resultSet = DadosfiscregraEntity::consultaDadoImpostoRegrafiscal($cdregrafisc, $cdimposto, $nrseqdadosfisc);
        return empty($resultSet);
    }

    /**
     * Verifica regras de origem/destino
     * Verifica se há duplicidade no registro a ser salvo.
     * @return array
     * @throws Exception
     */
    public function validaSave()
    {
        try{
            $resultSet = DadosfiscregraEntity::consultaDadosImpostoRegrafiscalNotThis($this->getCdregrafisc(), $this->getCdimposto(), $this->getNrseqdadosfisc());
            if(!empty($resultSet)) {
                $equals = array();
                foreach($resultSet as $row) {
                    // Já possui registro com opção Todos->Todos, quando cadastrado assim tabela só pode possuir 1 registro para respectiva regra fiscal.
                    if($row["IDTIPOORIGFISC"] === "T" && $row["IDTIPODESTFISC"] === "T") throw new Exception("Item padrão já cadastrado para o respectivo imposto.");
                    // Já possui registro cadastrado e usuário tenta inserir registro com opção Todos->Todos.
                    if($this->getIdtipoorigfisc() === "T" && $this->getIdtipodestfisc() === "T") throw new Exception("Item padrão já cadastrado para o respectivo imposto.");
                    // Se utilizar parametro de UF de origem
                    if($this->getValueFromSession("IDUTUFORIG") === "S") {
                        // Usuário tenta cadastrar registro porém já existe registro com origem Todos marcada.
                        if($row["IDTIPOORIGFISC"] === "T") throw new Exception("Item padrão já cadastrado para o respectivo imposto cujo valor independe da origem.");
                        // Se já possuo registro com origem espcifica e tento cadastrar um todos bloqueia.
                        if( $row["IDTIPOORIGFISC"] === "E" && $this->getIdtipoorigfisc() === "T") throw new Exception("Item padrão já cadastrado para o respectivo imposto com uma origem específica informada.");
                        $equals[0] = false;
                        // Se tipo de origem for especifica compara se há duplicidade de pais,estado,municipio.
                        if($this->getIdtipoorigfisc() === "E") {
                            if($row["CDPAISORIG"] === $this->getCdpaisorig() && $row["SGESTAORIG"] === $this->getSgestaorig() && $row["CDMUNIORIG"] === $this->getCdmuniorig()) {
                                $equals[0] = "Item padrão já cadastrado para o respectivo imposto com uma origem específica informada.";
                            }
                        }
                    }
                    // Se utilizar parametro de UF de destino
                    if($this->getValueFromSession("IDUTUFDEST") === "S") {
                        $equals[1] = false;
                        // Se tipo de destino for especifica compara se há duplicidade de pais,estado,municipio.
                        if($this->getIdtipodestfisc() === "E") {
                            if($row["CDPAISORIG"] === $this->getCdpaisdest() && $row["SGESTAORIG"] === $this->getSgestadest() && $row["CDMUNIORIG"] === $this->getCdmunidest()) {
                                $equals[1] = "Item padrão já cadastrado para o respectivo imposto com uma origem específica informada.";
                            }
                        }
                    }
                    if($this->getValueFromSession("IDUTFAIXVLR") === "S") {
                        $equals[2] = false;
                        if($row["VRINIFAIXA"] === $this->getVrinifaixa() && $row["VRFIMFAIXA"] === $this->getVrfimfaixa()) {
                            $equals[2] = "Registro com mesma faixa de valores já cadastrado";
                        }
                    }
                    // Usuário tenta cadastrar registro com IDCSTINTEREST/IDCSTINTERNA já cadastrado.
                    $equals[3] = false;
                    if($row["IDCSTINTEREST"] === $this->getIdcstinterest() || $row["IDCSTINTERNA"] === $this->getIdcstinterna()) {
                        $equals[3] = "Registro com abrangência do CST (estadual/interestadual ou internacional/nacional) já cadastrado.";
                    }
                    // Remove valores "false" do array deixando somente as mensagens de erro.
                    $errors = array_filter($equals);
                    if(!empty($errors)) return $errors;
                }
            }
            if($this->getCdfiscal() == 1) {
                if((float)$this->getVraliquota() < 0 || (float)$this->getVraliquota() > 100) {
                    throw new Exception("Valor da alíquota (%) deve estar entre 0 e 100,00%.");
                }
            }
            self::validaValorAliquota($this->getCdimposto(), $this->getCdsitutrib(), $this->getVraliquota());
            $this->_validaParametrizacao();
            $this->_validaOperacao();
        } catch(Exception $e) {
            throw $e;
        }
    }

    public static function validaValorAliquota($cdimposto, $cdsitutrib, $vraliquota)
    {
        try{
            $resultSet = AliquotacstnfeEntity::consultaAliquota($cdimposto, $cdsitutrib, $vraliquota);
            if(empty($resultSet)) throw new Exception("Alíquota inválida para o respectivo CST e imposto.");
        } catch(Exception $e) {
            throw $e;
        }
    }

    private function _validaParametrizacao()
    {
        $params = ParamcstEntity::consultaParametros($this->getCdimposto(), $this->getCdsitutrib());
        if($params instanceof ParamcstEntity) {
            // (caso a parametrização obrige informar a natureza da receita para o cst e imposto.
            if ($params->getIdutnatureceita() === "S") {
                if ($this->getCdnaturec() === "") throw new Exception("Natureza da Receita deve ser informada.");
                if ($this->getCdtpcrpicostnfe() === "") throw new Exception("Tipo de Crédito deve ser informado.");
                if ($this->getCdnatubcpcstnfe() === "") throw new Exception("Natureza da Base de Cálculo deve ser informada.");
            }
            if ($params->getIdutialiqqtcst() === "N") throw new Exception("CST inválido para imposto cujo valor de alíquota é definido por quantidade.");
            $validCdfisc = array();
            if($params->getIduticdfiscal1() === "S") $validCdfisc[] = 1;
            if($params->getIduticdfiscal2() === "S") $validCdfisc[] = 2;
            if($params->getIduticdfiscal3() === "S") $validCdfisc[] = 3;
            if(!in_array((int)$this->getCdfiscal(), $validCdfisc)) throw new Exception("Código Fiscal inválido para o respectivo CST e imposto.");
        }
    }

    private function _validaOperacao()
    {
        $results = array();
        if($this->getIdtipoorigfisc() === "E"){
            $results[] = $this->getCdpaisorig() === "" ? "País de origem não pode ser nulo." : false;
            $results[] = $this->getSgestaorig() === "" ? "Estado de origem não pode ser nulo." : false;
            $results[] = $this->getCdmuniorig() === "" ? "Município de origem não pode ser nulo." : false;
        }
        if($this->getIdtipodestfisc() === "E"){
            $results[] = $this->getCdpaisdest() === "" ? "País de destino não pode ser nulo." : false;
            $results[] = $this->getSgestadest() === "" ? "Estado de destino não pode ser nulo." : false;
            $results[] = $this->getCdmunidest() === "" ? "Município de destino não pode ser nulo." : false;
        }
        $errors = array_filter($results);
        if(!empty($errors)) throw new Exception(implode("\n", $errors));

        if ($this->getIdtipoorigfisc() === $this->getIdtipodestfisc() && $this->getIdtipoorigfisc() === "E") {
            if ($this->getIdcstinterest() === "I" && $this->getSgestaorig() === $this->getSgestadest()) {
                throw new Exception("Estado origem e destino devem ser diferentes para operações interestaduais.");
            } elseif($this->getIdcstinterest() === "E" && $this->getSgestaorig() !== $this->getSgestadest()) {
                throw new Exception("Estado origem e destino devem iguais para operações estaduais.");
            }
            if ($this->getIdcstinterna() === "I" && $this->getCdpaisorig() === $this->getCdpaisdest()) {
                throw new Exception("País origem e destino devem ser diferentes para operações internacionais.");
            } elseif($this->getIdcstinterna() === "N" && $this->getCdpaisorig() !== $this->getCdpaisdest()) {
                throw new Exception("Estado origem e destino devem iguais para operações nacionais.");
            }
        }

        return true;
    }


}