/* Sistemas Integrados TEKNISA
  Documentos Fiscais

Nome: 5550_DFESTP0_GERA_RE.SQL

@C:\Web_622\Sql\End_Conv\Oracle\5550_DFESTPO_GERA_RE.sql

Descrio:
Rotina para gerao da capa do documento (RE).

P_CDFILIAL      - Cdigo da filial a ser gerado RE.
P_IDENTRSAID    - Identificador de 'E' - Entrada e 'S' - Sada
P_DATARE        - Data da gerao do RE
P_TIPOEXPRE     - Tipo de RE ('F' - Filial e 'C' - Centralizadora)
P_CDALMOXARIFE  - Almoxarifado
P_CDLOCALESTOQ  - Localizao
P_IDERRO        - Parmetro de retorno para indicar se o RE de entrada j foi emitido.

PARA DEBUGAR
-- SET SERVEROUTPUT ON
-- DBMS_OUTPUT.PUT_LINE('xx');
-- raise_application_error(-20501, '' , True);

Criao:
08/03/2010 - Fabrcio Marcelo da Silva
*/

CREATE OR REPLACE PROCEDURE GERA_RE(
     P_CDFILIAL      IN FILIAL.CDFILIAL%TYPE,
     P_IDENTRSAID    IN NOTAFISCAL.IDENTRSAIDOP%TYPE,
     P_DATARE        IN NOTAFISCAL.DTENTRSAID%TYPE,
     P_TIPOEXPRE     IN NOTAFISCAL.IDSTATUS%TYPE,
     P_CDALMOXARIFE  IN ALMOXARIFADO.CDALMOXARIFE%TYPE,
     P_CDLOCALESTOQ  IN LOCALESTOQUE.CDLOCALESTOQ%TYPE,
     P_IDERRO        IN OUT VARCHAR2)
AS
   V_DTPROCESSA       DATE;
   V_IDREINICIANUMRE  CHAR(1) := 'N';  -- Identificador reiniciar nmerao de RE
   V_IDPARAMEMISRE    CHAR(1) := 'N';  -- Identificador para considerar parametrizao para emisso do RE
   V_CDPARAMEMISRE    VARCHAR2(10);
   V_IDCAPACORRES     CHAR(1) := 'S';  -- Identificador se possui capa correspondente conforme parametrizao

   V_ANTNUMRE         VARCHAR2(20);
   V_NUMERORE         VARCHAR2(10);
   V_PRXNUMRE         VARCHAR2(20);
   V_NRREPOST         VARCHAR2(10);

   V_IDSEQCRIADO      CHAR(1) := 'N';  -- Identificador de sequencial RE criado

   V_CDMODEDOCU       VARCHAR2(200);
   V_CDTIPOOPER       VARCHAR2(200);
   V_CDFORNECED       VARCHAR2(1000);
   V_CDCLIENTE        VARCHAR2(1000);

   V_DTFECMES         DATE;
   V_NRMAIORRE        VARCHAR2(20);
   V_REGERADOE        VARCHAR2(04);

   V_CDTIPOFATUR      VARCHAR2(03);
   V_CDINSCESTA       VARCHAR2(16);
   V_CDEMPRESA        VARCHAR2(02);

   V_PCDTIPOCONT      VARCHAR2(03);
   V_PCDINSCESTA      VARCHAR2(16);
   V_PCDEMPRESA       VARCHAR2(02);

   V_NUMRENANO        VARCHAR2(20);
   V_NUMRENPER        VARCHAR2(20);
   V_DTFECMESANT      DATE;

   V_REGERADO         VARCHAR2(01);
   V_NRREAUX          VARCHAR2(20);
   V_CDREGRA          VARCHAR2(10);
   V_NUMERORED        VARCHAR2(10);

   /* Regras disponveis para a filial para gerao da capa de documento (RE) */
   CURSOR C_PARAMEMISSAO IS
      SELECT CDPARAMEMISRE, DSOBPARAMEMISRE
        FROM PARAMEMISSAORE
       WHERE NVL(CDTIPOFATEMISRE, 'T') = V_PCDTIPOCONT
         AND CDEMPRESA = V_PCDEMPRESA
         AND NVL(CDINSCESTA, 'T') = V_PCDINSCESTA
         AND IDENTSAIDEMISRE = P_IDENTRSAID
         AND IDSITUQUEBRARE = 'A'
       ORDER BY CDPRIOEMISSAORE;

   /* Notas fiscais de entrada */
   CURSOR C_NOTAFISCALE IS
      SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, NVL(N.CDFORNECED, '*') AS CDFORNECED, NVL(N.CDCLIENTE, '*') AS CDCLIENTE
        FROM NOTAFISCAL N, ITEMPRODUTO I, TIPOOPERAC T, ALMOXARIFADO A,
             LOCALESTOQUE L
       WHERE N.CDFILIRECE = P_CDFILIAL
         AND N.IDENTRSAIDOP = 'E'
         AND N.DTENTRSAID = P_DATARE
         AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
         AND N.IDSTATUS IN ('P','A','E')
         AND N.CDTIPOOPER NOT IN ('10', '09')
         AND N.NRSEQCAPADOC IS NULL
         AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
         AND N.CDTIPOOPER = T.CDTIPOOPER
         AND NVL(T.IDCOMPEMISRE, 'F') = 'F'
         AND I.CDFILIAL = N.CDFILIAL
         AND I.NRLANCTONF = N.NRLANCTONF
         AND ((I.CDALMOXARIFE = P_CDALMOXARIFE) OR (P_CDALMOXARIFE = 'T'))
         AND ((I.CDLOCALESTOQ = P_CDLOCALESTOQ) OR (P_CDLOCALESTOQ = 'T'))
         AND I.CDFILIAL = A.CDFILIAL(+)
         AND I.CDALMOXARIFE = A.CDALMOXARIFE(+)
         AND I.CDFILIAL = L.CDFILIAL(+)
         AND I.CDALMOXARIFE = L.CDALMOXARIFE(+)
         AND I.CDLOCALESTOQ = L.CDLOCALESTOQ(+);

   /* Notas fiscais de sada */
   CURSOR C_NOTAFISCALS IS
      SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, NVL(N.CDFORNECED, '*') AS CDFORNECED, NVL(N.CDCLIENTE, '*') AS CDCLIENTE
        FROM NOTAFISCAL N, ITEMPRODUTO I, TIPOOPERAC T, ALMOXARIFADO A,
             LOCALESTOQUE L
       WHERE N.CDFILIEMIT = P_CDFILIAL
         AND N.IDENTRSAIDOP = 'S'
         AND N.DTENTRSAID = P_DATARE
         AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
         AND N.IDSTATUS NOT IN ('C', 'D')
         AND N.NRSEQCAPADOC IS NULL
         AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
         AND N.CDTIPOOPER = T.CDTIPOOPER
         AND NVL(T.IDCOMPEMISRE, 'F') = 'F'
         AND I.CDFILIAL = N.CDFILIAL
         AND I.NRLANCTONF = N.NRLANCTONF
         AND ((I.CDALMOXARIFE = P_CDALMOXARIFE) OR (P_CDALMOXARIFE = 'T'))
         AND ((I.CDLOCALESTOQ = P_CDLOCALESTOQ) OR (P_CDLOCALESTOQ = 'T'))
         AND I.CDFILIAL = A.CDFILIAL(+)
         AND I.CDALMOXARIFE = A.CDALMOXARIFE(+)
         AND I.CDFILIAL = L.CDFILIAL(+)
         AND I.CDALMOXARIFE = L.CDALMOXARIFE(+)
         AND I.CDLOCALESTOQ = L.CDLOCALESTOQ(+)
         AND (NOT EXISTS (SELECT P_CDFILIAL
                            FROM FILIAL FL
                           WHERE FL.CDFILIAL = N.CDFILIEMIT
                             AND FL.CDFILIFISC <> N.CDFILIEMIT
                             AND FL.CDFILIFISC IS NOT NULL
                             AND N.CDTIPOOPER IN ('01','12','13')));

   /* Notas fiscais de sada - RE Centralizadora */
   CURSOR C_NOTAFISCALSC IS
      SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, NVL(N.CDFORNECED, '*') AS CDFORNECED, NVL(N.CDCLIENTE, '*') AS CDCLIENTE
        FROM NOTAFISCAL N, ALMOXARIFADO A, LOCALESTOQUE L, FILIAL F,
             FILIAL I, TIPOOPERAC T
       WHERE N.IDENTRSAIDOP = 'S'
         AND IDSTATUS IN ('A','E')
         AND N.DTAPROVACAO   = P_DATARE
         AND N.NRSEQCAPADOC IS NULL
         AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
         AND ((N.CDALMOXARIFE = NVL(P_CDALMOXARIFE, N.CDALMOXARIFE)) OR P_CDALMOXARIFE = 'T')
         AND ((N.CDLOCALESTOQ = NVL(P_CDLOCALESTOQ, N.CDLOCALESTOQ)) OR P_CDLOCALESTOQ = 'T')
         AND N.CDALMOXARIFE = A.CDALMOXARIFE(+)
         AND N.CDFILIAL = A.CDFILIAL(+)
         AND N.CDALMOXARIFE = L.CDALMOXARIFE(+)
         AND N.CDFILIAL = L.CDFILIAL(+)
         AND N.CDLOCALESTOQ = L.CDLOCALESTOQ(+)
         AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
         AND N.CDTIPOOPER = T.CDTIPOOPER
         AND T.IDCOMPEMISRE = 'C'
         AND N.CDFILIEMIT = F.CDFILIAL
         AND P_CDFILIAL = I.CDFILIAL
         AND (EXISTS (SELECT CDFILIAL
                        FROM FILIAL FL
                       WHERE NVL(CDFILIFISC,FL.CDFILIAL) = P_CDFILIAL
                         AND N.CDFILIEMIT = FL.CDFILIAL));
BEGIN
   P_IDERRO := 'N';

   /* Verifica a data de processamento da filial, se ser reiniciado a nmerao do RE
      e se a filial considera parametrizao para emisso do RE */
   BEGIN
      SELECT PF.DTPROCESSA, PF.IDREINICIANUMRE, DECODE(PF.IDPARAMEMISREFI, 'N', P.IDPARAMEMISRE, 'S') AS IDPARAMEMISRE
        INTO V_DTPROCESSA, V_IDREINICIANUMRE, V_IDPARAMEMISRE
        FROM PARAMFILIAL PF, PARAMGERAL P
       WHERE PF.CDFILIAL = P_CDFILIAL
         AND P.CDPARGERAL = '0001';
   EXCEPTION
        WHEN NO_DATA_FOUND THEN
           V_DTPROCESSA := NULL;
           V_IDREINICIANUMRE := 'N';
           V_IDPARAMEMISRE := 'N';
   END;

   /* Verifica se existe parametrizao para emisso do RE que atenda a filial
      a ser gerada a capa de documento (RE) */
   BEGIN
      SELECT PF.CDTIPOFATUR, F.CDINSCESTA, F.CDEMPRESA
        INTO V_CDTIPOFATUR, V_CDINSCESTA, V_CDEMPRESA
        FROM FILIAL F, PARAMFILIAL PF
       WHERE F.CDFILIAL = P_CDFILIAL
         AND PF.CDFILIAL = P_CDFILIAL;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN
           V_CDTIPOFATUR := NULL; V_CDINSCESTA := NULL; V_CDEMPRESA := NULL;
   END;

   BEGIN
      SELECT MAX(CDPARAMEMISRE)
        INTO V_CDPARAMEMISRE
        FROM PARAMEMISSAORE P
       WHERE P.CDTIPOFATEMISRE = V_CDTIPOFATUR
         AND P.CDEMPRESA = V_CDEMPRESA
         AND P.CDINSCESTA = V_CDINSCESTA
         AND P.IDENTSAIDEMISRE = P_IDENTRSAID
         AND P.IDSITUQUEBRARE = 'A';
   EXCEPTION
        WHEN NO_DATA_FOUND THEN
           V_CDPARAMEMISRE := NULL;
   END;

   IF V_CDPARAMEMISRE IS NULL THEN
      BEGIN
         SELECT MAX(CDPARAMEMISRE)
           INTO V_CDPARAMEMISRE
           FROM PARAMEMISSAORE P
          WHERE P.CDTIPOFATEMISRE = V_CDTIPOFATUR
            AND P.CDEMPRESA = V_CDEMPRESA
            AND P.CDINSCESTA IS NULL
            AND P.IDENTSAIDEMISRE = P_IDENTRSAID
            AND P.IDSITUQUEBRARE = 'A';
      EXCEPTION
           WHEN NO_DATA_FOUND THEN
              V_CDPARAMEMISRE := NULL;
      END;

      IF V_CDPARAMEMISRE IS NULL THEN
         BEGIN
            SELECT MAX(CDPARAMEMISRE)
              INTO V_CDPARAMEMISRE
              FROM PARAMEMISSAORE P
             WHERE P.CDTIPOFATEMISRE IS NULL
               AND P.CDEMPRESA = V_CDEMPRESA
               AND P.CDINSCESTA = V_CDINSCESTA
               AND P.IDENTSAIDEMISRE = P_IDENTRSAID
               AND P.IDSITUQUEBRARE = 'A';
         EXCEPTION
              WHEN NO_DATA_FOUND THEN
                 V_CDPARAMEMISRE := NULL;
         END;

         IF V_CDPARAMEMISRE IS NULL THEN
            BEGIN
               SELECT MAX(CDPARAMEMISRE)
                 INTO V_CDPARAMEMISRE
                 FROM PARAMEMISSAORE P
                WHERE P.CDTIPOFATEMISRE IS NULL
                  AND P.CDEMPRESA = V_CDEMPRESA
                  AND P.CDINSCESTA IS NULL
                  AND P.IDENTSAIDEMISRE = P_IDENTRSAID
                  AND P.IDSITUQUEBRARE = 'A';
            EXCEPTION
                 WHEN NO_DATA_FOUND THEN
                    V_CDPARAMEMISRE := NULL;
            END;

            IF V_CDPARAMEMISRE IS NULL THEN
               V_PCDTIPOCONT := NULL;
               V_PCDEMPRESA  := V_CDEMPRESA;
               V_PCDINSCESTA := NULL;
            ELSE
               V_PCDTIPOCONT := 'T';
               V_PCDEMPRESA  := V_CDEMPRESA;
               V_PCDINSCESTA := 'T';
            END IF;
         ELSE
            V_PCDTIPOCONT := 'T';
            V_PCDEMPRESA  := V_CDEMPRESA;
            V_PCDINSCESTA := V_CDINSCESTA;
         END IF;
      ELSE
         V_PCDTIPOCONT := V_CDTIPOFATUR;
         V_PCDEMPRESA  := V_CDEMPRESA;
         V_PCDINSCESTA := 'T';
      END IF;
   ELSE
      V_PCDTIPOCONT := V_CDTIPOFATUR;
      V_PCDEMPRESA  := V_CDEMPRESA;
      V_PCDINSCESTA := V_CDINSCESTA;
   END IF;


   /* RE de notas fiscais (NORMAL) */
   IF P_TIPOEXPRE = 'F' THEN
      /* Notas fiscais de Entrada */
      IF P_IDENTRSAID = 'E' THEN

         V_REGERADO := 'N';

         /* Verifica se possui algum RE posterior a data do RE gerado */
         SELECT MIN(NRNUMERNF)
           INTO V_NRREPOST
           FROM NUMERORNF
          WHERE CDFILIAL = P_CDFILIAL
            AND DTNUMERNF > P_DATARE;

         IF V_NRREPOST IS NULL THEN

            /* Verifica se a data do RE  igual a data de processamento da filial,
               caso seja, ser gerado novamente o RE de notas fiscais */

            IF V_DTPROCESSA = P_DATARE THEN
               V_NRREAUX := NULL;
               SELECT MIN(NRSEQCAPADOC)
                 INTO V_NRREAUX
                 FROM NOTAFISCAL
                WHERE CDFILIRECE = P_CDFILIAL
                  AND IDENTRSAIDOP = P_IDENTRSAID
                  AND DTENTRSAID = P_DATARE
                  AND (IDSISTEMA IN ('EST','DFE') OR IDSISTEMA IS NULL);


               UPDATE NOTAFISCAL
                  SET CDPARAMEMISRE = NULL,
                      NRSEQCAPADOC = NULL
                WHERE CDFILIRECE = P_CDFILIAL
                  AND IDENTRSAIDOP = P_IDENTRSAID
                  AND DTENTRSAID = P_DATARE
                  AND (IDSISTEMA IN ('EST','DFE') OR IDSISTEMA IS NULL);

               UPDATE NOTAFISCAL
                  SET CDPARAMEMISRE = NULL,
                      NRSEQCAPADOC = NULL
                WHERE CDFILIEMIT = P_CDFILIAL
                  AND IDENTRSAIDOP = 'S'
                  AND DTENTRSAID = P_DATARE
                  AND (IDSISTEMA IN ('EST','DFS','FAI') OR IDSISTEMA IS NULL);

               DELETE FROM NUMERORNF
                WHERE CDFILIAL = P_CDFILIAL
                  AND DTNUMERNF = P_DATARE;

               /* Retorna sequencial do dia ltimo dia gerado, caso seja cancelado o RE do dia */
               BEGIN
                  SELECT LPAD(MAX(NRNUMERNF),20,'0')
                    INTO V_ANTNUMRE
                    FROM NUMERORNF
                   WHERE CDFILIAL = P_CDFILIAL
                     AND DTNUMERNF = (SELECT MAX(DTNUMERNF)
                                        FROM NUMERORNF
                                       WHERE CDFILIAL = P_CDFILIAL
                                         AND DTNUMERNF <= (P_DATARE - 1));
               EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                       V_ANTNUMRE := NULL;
               END;

               IF V_ANTNUMRE IS NULL THEN
                 IF NOT (V_NRREAUX IS NULL) THEN
                  V_ANTNUMRE := TO_CHAR(TO_NUMBER(V_NRREAUX) - 1);	
                 END IF;
               END IF;
            
               IF V_ANTNUMRE IS NOT NULL THEN               
                    UPDATE NOVOCODIGO
                  	SET NRSEQUENCIAL = V_ANTNUMRE
                		WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
              END IF;
            END IF;
         ELSE
            /* Verifica se j possui RE (Capa de documento) gerada para o dia */
            BEGIN
               SELECT MAX(N.NRSEQCAPADOC)
                 INTO V_NUMERORE
                 FROM NOTAFISCAL N
                WHERE N.CDFILIRECE = P_CDFILIAL
                  AND N.IDENTRSAIDOP = 'E'
                  AND N.IDSTATUS IN ('P','A','E')
                  AND N.DTENTRSAID = P_DATARE
                  AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                  AND NOT N.NRSEQCAPADOC IS NULL;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_NUMERORE := NULL;
            END;

            IF NOT V_NUMERORE IS NULL THEN
               BEGIN
                  SELECT MAX(N.CDPARAMEMISRE)
                    INTO V_CDREGRA
                    FROM NOTAFISCAL N
                   WHERE N.CDFILIRECE = P_CDFILIAL
                     AND N.IDENTRSAIDOP = 'E'
                     AND N.IDSTATUS IN ('P','A','E')
                     AND N.DTENTRSAID = P_DATARE
                     AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                     AND N.NRSEQCAPADOC = V_NUMERORE;
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     V_CDREGRA := NULL;
               END;

               FOR C_NOTAFISCAL IN C_NOTAFISCALE LOOP

                  UPDATE NOTAFISCAL
                     SET NRSEQCAPADOC = V_NUMERORE,
                         CDPARAMEMISRE = V_CDREGRA
                   WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                     AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
               END LOOP;
            END IF;

            V_REGERADO := 'S';
         END IF;

         IF V_REGERADO = 'N' THEN

            /* Verifica a data de fechamento do ms de dezembro */
            BEGIN
               SELECT MAX(NRNUMERNF)
                 INTO V_NUMRENANO
                 FROM NUMERORNF
                WHERE CDFILIAL = P_CDFILIAL
                  AND DTNUMERNF >= TO_DATE('01/01/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
            EXCEPTION
                 WHEN NO_DATA_FOUND THEN
                    V_NUMRENANO := NULL;
            END;

            BEGIN
               SELECT DTFECMES + 1
                 INTO V_DTFECMESANT
                 FROM DIAFECMES
                WHERE CDFILIAL = P_CDFILIAL
                  AND DTFECMES BETWEEN TO_DATE('01/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1)) AND TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1));
            EXCEPTION
                 WHEN NO_DATA_FOUND THEN
                    V_DTFECMESANT := TO_DATE('01/01/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
            END;

            BEGIN
               SELECT MAX(NRNUMERNF)
                 INTO V_NUMRENPER
                 FROM NUMERORNF
                WHERE CDFILIAL = P_CDFILIAL
                  AND DTNUMERNF BETWEEN V_DTFECMESANT AND P_DATARE;
            EXCEPTION
                 WHEN NO_DATA_FOUND THEN
                    V_NUMRENPER := NULL;
            END;

            IF (V_NUMRENANO IS NULL  AND
                V_NUMRENPER IS NULL) THEN
               BEGIN
                  SELECT DTFECMES
                    INTO V_DTFECMES
                    FROM DIAFECMES
                   WHERE CDFILIAL = P_CDFILIAL
                     AND DTFECMES BETWEEN TO_DATE('01/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1)) AND TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1));
               EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                       V_DTFECMES := TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1));
               END;
            ELSE
               BEGIN
                  SELECT DTFECMES
                    INTO V_DTFECMES
                    FROM DIAFECMES
                   WHERE CDFILIAL = P_CDFILIAL
                     AND DTFECMES BETWEEN TO_DATE('01/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY')))) AND TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
               EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                       V_DTFECMES := TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
               END;
            END IF;

            /* Verifica se existe algum RE posterior a data de fechamento do ms de dezembro do ano anterior */
            BEGIN
               SELECT MAX(NRNUMERNF)
                 INTO V_NRMAIORRE
                 FROM NUMERORNF
                WHERE CDFILIAL = P_CDFILIAL
                  AND DTNUMERNF <> P_DATARE
                  AND DTNUMERNF > V_DTFECMES;
            EXCEPTION
                 WHEN NO_DATA_FOUND THEN
                    V_NRMAIORRE := NULL;
            END;


            /* Utiliza parametrizao para emisso do RE (Regras) e possui regras */
            IF (V_IDPARAMEMISRE = 'S' AND NOT V_CDPARAMEMISRE IS NULL) THEN

               FOR C_PARAMEMIS IN C_PARAMEMISSAO LOOP

                  V_IDSEQCRIADO := 'N';

                  V_CDMODEDOCU := ' ';
                  V_CDTIPOOPER := ' ';
                  V_CDFORNECED := ' ';
                  V_CDCLIENTE  := ' ';

                  FOR C_MODEDOCU IN (SELECT CDMODDOCFISC
                                       FROM MODDOCEMISSAORE
                                      WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE) LOOP
                     V_CDMODEDOCU := V_CDMODEDOCU || C_MODEDOCU.CDMODDOCFISC || '|';
                  END LOOP;
                  FOR C_TIPOOPER IN (SELECT CDTIPOOPER
                                       FROM TIPOPERACEMISRE
                                      WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE
                                        AND IDENTRSAIDOP = P_IDENTRSAID) LOOP
                     V_CDTIPOOPER := V_CDTIPOOPER || C_TIPOOPER.CDTIPOOPER || '|';
                  END LOOP;
                  FOR C_FORNECED IN (SELECT CDFORNECED
                                       FROM FORNECEMISSAORE
                                      WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE) LOOP
                     V_CDFORNECED := V_CDFORNECED || C_FORNECED.CDFORNECED || '|';
                  END LOOP;
                  FOR C_CLIENTE IN (SELECT CDCLIENTE
                                      FROM CLIENTEMISSAORE
                                     WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE) LOOP
                     V_CDCLIENTE := V_CDCLIENTE || C_CLIENTE.CDCLIENTE || '|';
                  END LOOP;

                  FOR C_NOTAFISCAL IN C_NOTAFISCALE LOOP

                     V_IDCAPACORRES := 'S';

                     IF (NOT V_CDMODEDOCU = ' ' AND V_IDCAPACORRES = 'S') THEN
                        IF INSTR(V_CDMODEDOCU, C_NOTAFISCAL.CDMODDOCFISC) = 0 THEN
                           V_IDCAPACORRES := 'N';
                        END IF;
                     END IF;

                     IF (NOT V_CDTIPOOPER = ' ' AND V_IDCAPACORRES = 'S') THEN
                        IF INSTR(V_CDTIPOOPER, C_NOTAFISCAL.CDTIPOOPER) = 0 THEN
                           V_IDCAPACORRES := 'N';
                        END IF;
                     END IF;

                     IF (NOT V_CDFORNECED = ' ' AND V_IDCAPACORRES = 'S') OR
                        (NOT V_CDCLIENTE = ' ' AND V_IDCAPACORRES = 'S')  THEN
                        IF ((INSTR(V_CDFORNECED, C_NOTAFISCAL.CDFORNECED) = 0) AND
                            (INSTR(V_CDCLIENTE, C_NOTAFISCAL.CDCLIENTE) = 0))  THEN
                           V_IDCAPACORRES := 'N';
                        END IF;
                     END IF;

                     /* Se atende as regras de capa de documento conforme parametrizao (regras) */
                     IF V_IDCAPACORRES = 'S' THEN
                        /* Verifica se j possui RE (Capa de documento) gerada para o dia conforme regras */
                        BEGIN
                           SELECT MAX(N.NRSEQCAPADOC)
                             INTO V_NUMERORE
                             FROM NOTAFISCAL N
                            WHERE N.CDFILIRECE = P_CDFILIAL
                              AND N.IDENTRSAIDOP = 'E'
                              AND N.IDSTATUS IN ('P','A','E')
                              AND N.DTENTRSAID = P_DATARE
                              AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                              AND NOT N.NRSEQCAPADOC IS NULL
                              AND N.CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE;
                        EXCEPTION
                           WHEN NO_DATA_FOUND THEN
                              V_NUMERORE := NULL;
                        END;

                        IF V_NUMERORE IS NULL THEN
                           IF V_IDSEQCRIADO <> 'S' THEN
                              IF ((V_IDREINICIANUMRE = 'S') AND
                                  (P_DATARE > V_DTFECMES)   AND
                                  (V_NRMAIORRE IS NULL))    THEN

                                 /* Verifica se j possui RE (Capa de documento) gerada para o dia independente de regras */
                                 BEGIN
                                    SELECT MAX(N.NRSEQCAPADOC)
                                      INTO V_NUMERORED
                                      FROM NOTAFISCAL N
                                     WHERE N.CDFILIRECE = P_CDFILIAL
                                       AND N.IDENTRSAIDOP = 'E'
                                       AND N.IDSTATUS IN ('P','A','E')
                                       AND N.DTENTRSAID = P_DATARE
                                       AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                                       AND NOT N.NRSEQCAPADOC IS NULL;
                                 EXCEPTION
                                    WHEN NO_DATA_FOUND THEN
                                       V_NUMERORED := NULL;
                                 END;

                                 IF V_NUMERORED IS NULL THEN
                                    UPDATE NOVOCODIGO
                                       SET NRSEQUENCIAL = LPAD('0',20,'0')
                                     WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
                                 END IF;
                              END IF;

                              V_PRXNUMRE := NULL;

                              NOVO_CODIGO('RENOTAFISCAL'||P_CDFILIAL, V_PRXNUMRE, 1);
                              V_NUMERORE := SUBSTR(V_PRXNUMRE, 11, 10);

                              /* Insere novo nmero de RE */
                              INSERT INTO NUMERORNF
                                (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                              VALUES
                                (P_CDFILIAL, P_DATARE, P_IDENTRSAID, P_IDENTRSAID, V_NUMERORE);

                              UPDATE PARAMFILIAL
                                 SET NRRENOTAFISC = V_NUMERORE
                               WHERE CDFILIAL = P_CDFILIAL;

                              V_IDSEQCRIADO := 'S';
                           END IF;
                        END IF;
                        UPDATE NOTAFISCAL
                           SET CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE,
                               NRSEQCAPADOC = V_NUMERORE
                         WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                           AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
                     END IF;
                  END LOOP;
               END LOOP;

               /*   Verifica notas que no possuem nenhuma regra e gera uma capa de documento (maior capa do dia) */
               V_IDSEQCRIADO := 'N';

               FOR C_NOTAFISCAL IN C_NOTAFISCALE LOOP
                  IF V_IDSEQCRIADO <> 'S' THEN
                     IF ((V_IDREINICIANUMRE = 'S') AND
                         (P_DATARE > V_DTFECMES)   AND
                         (V_NRMAIORRE IS NULL))    THEN

                        /* Verifica se j possui RE (Capa de documento) gerada para o dia */
                        BEGIN
                           SELECT MAX(NRNUMERNF)
                             INTO V_NUMERORE
                             FROM NUMERORNF
                            WHERE CDFILIAL = P_CDFILIAL
                              AND DTNUMERNF = P_DATARE
                              AND IDENTRSAID = 'E'
                              AND IDTIPOOPERNF = 'E';
                        EXCEPTION
                           WHEN NO_DATA_FOUND THEN
                              V_NUMERORE := NULL;
                        END;

                        IF V_NUMERORE IS NULL THEN
                           UPDATE NOVOCODIGO
                              SET NRSEQUENCIAL = LPAD('0',20,'0')
                            WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
                        END IF;
                     END IF;

                     V_PRXNUMRE := NULL;

                     NOVO_CODIGO('RENOTAFISCAL'||P_CDFILIAL, V_PRXNUMRE, 1);
                     V_NUMERORE := SUBSTR(V_PRXNUMRE, 11, 10);

                     /* Insere novo nmero de RE */
                     INSERT INTO NUMERORNF
                       (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                     VALUES
                       (P_CDFILIAL, P_DATARE, P_IDENTRSAID, P_IDENTRSAID, V_NUMERORE);

                     UPDATE PARAMFILIAL
                        SET NRRENOTAFISC = V_NUMERORE
                      WHERE CDFILIAL = P_CDFILIAL;

                     V_IDSEQCRIADO := 'S';
                  END IF;

                  UPDATE NOTAFISCAL
                     SET NRSEQCAPADOC = V_NUMERORE
                   WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                     AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
               END LOOP;
            ELSE
               /* Verifica se j possui RE (Capa de documento) gerada para o dia */
               BEGIN
                  SELECT MAX(N.NRSEQCAPADOC)
                    INTO V_NUMERORE
                    FROM NOTAFISCAL N
                   WHERE N.CDFILIRECE = P_CDFILIAL
                     AND N.IDENTRSAIDOP = 'E'
                     AND N.IDSTATUS IN ('P','A','E')
                     AND N.DTENTRSAID = P_DATARE
                     AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                     AND NOT N.NRSEQCAPADOC IS NULL;
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     V_NUMERORE := NULL;
               END;

               BEGIN
                  SELECT MAX(N.CDPARAMEMISRE)
                    INTO V_CDREGRA
                    FROM NOTAFISCAL N
                   WHERE N.CDFILIRECE = P_CDFILIAL
                     AND N.IDENTRSAIDOP = 'E'
                     AND N.IDSTATUS IN ('P','A','E')
                     AND N.DTENTRSAID = P_DATARE
                     AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                     AND N.NRSEQCAPADOC = V_NUMERORE;
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     V_CDREGRA := NULL;
               END;

               V_IDSEQCRIADO := 'N';

               FOR C_NOTAFISCAL IN C_NOTAFISCALE LOOP

                  IF V_NUMERORE IS NULL THEN
                     IF V_IDSEQCRIADO <> 'S' THEN
                        IF ((V_IDREINICIANUMRE = 'S') AND
                            (P_DATARE > V_DTFECMES)   AND
                            (V_NRMAIORRE IS NULL))    THEN
                           UPDATE NOVOCODIGO
                              SET NRSEQUENCIAL = LPAD('0',20,'0')
                            WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
                        END IF;

                        V_PRXNUMRE := NULL;

                        NOVO_CODIGO('RENOTAFISCAL'||P_CDFILIAL, V_PRXNUMRE, 1);
                        V_NUMERORE := SUBSTR(V_PRXNUMRE, 11, 10);

                        /* Insere novo nmero de RE */
                        INSERT INTO NUMERORNF
                          (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                        VALUES
                          (P_CDFILIAL, P_DATARE, P_IDENTRSAID, P_IDENTRSAID, V_NUMERORE);

                        UPDATE PARAMFILIAL
                           SET NRRENOTAFISC = V_NUMERORE
                         WHERE CDFILIAL = P_CDFILIAL;

                        V_IDSEQCRIADO := 'S';
                     END IF;
                  END IF;

                  UPDATE NOTAFISCAL
                     SET NRSEQCAPADOC = V_NUMERORE,
                         CDPARAMEMISRE = V_CDREGRA
                   WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                     AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
               END LOOP;
            END IF;
         END IF;
      ELSE
         /* Gerao de RE de Sada */

         BEGIN
            SELECT MAX(N.CDFILIAL)
              INTO V_REGERADOE
              FROM NOTAFISCAL N, ITEMPRODUTO I, TIPOOPERAC T, ALMOXARIFADO A,
                   LOCALESTOQUE L
             WHERE N.CDFILIRECE = P_CDFILIAL
               AND N.IDENTRSAIDOP = 'E'
               AND N.DTENTRSAID = P_DATARE
               AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
               AND N.IDSTATUS IN ('P','A','E')
               AND N.CDTIPOOPER NOT IN ('10', '09')
               AND N.NRSEQCAPADOC IS NULL
               AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
               AND N.CDTIPOOPER = T.CDTIPOOPER
               AND NVL(T.IDCOMPEMISRE, 'F') = 'F'
               AND I.CDFILIAL = N.CDFILIAL
               AND I.NRLANCTONF = N.NRLANCTONF
               AND ((I.CDALMOXARIFE = P_CDALMOXARIFE) OR (P_CDALMOXARIFE = 'T'))
               AND ((I.CDLOCALESTOQ = P_CDLOCALESTOQ) OR (P_CDLOCALESTOQ = 'T'))
               AND I.CDFILIAL = A.CDFILIAL(+)
               AND I.CDALMOXARIFE = A.CDALMOXARIFE(+)
               AND I.CDFILIAL = L.CDFILIAL(+)
               AND I.CDALMOXARIFE = L.CDALMOXARIFE(+)
               AND I.CDLOCALESTOQ = L.CDLOCALESTOQ(+);
         EXCEPTION
              WHEN NO_DATA_FOUND THEN
                 V_REGERADOE := NULL;
         END;

         IF V_REGERADOE IS NULL THEN

            V_REGERADO := 'N';

            /* Verifica se possui algum RE posterior a data do RE gerado */
            SELECT MIN(NRNUMERNF)
              INTO V_NRREPOST
              FROM NUMERORNF
             WHERE CDFILIAL = P_CDFILIAL
               AND DTNUMERNF > P_DATARE;

            IF V_NRREPOST IS NULL THEN

               /* Verifica se a data do RE  igual a data de processamento da filial,
                  caso seja, ser gerado novamente o RE de notas fiscais */
               IF V_DTPROCESSA = P_DATARE THEN
                  UPDATE NOTAFISCAL
                     SET CDPARAMEMISRE = NULL,
                         NRSEQCAPADOC = NULL
                   WHERE CDFILIEMIT = P_CDFILIAL
                     AND IDENTRSAIDOP = P_IDENTRSAID
                     AND DTENTRSAID = P_DATARE
                     AND (IDSISTEMA IN ('EST','DFS','FAI') OR IDSISTEMA IS NULL);

                  DELETE FROM NUMERORNF
                   WHERE CDFILIAL = P_CDFILIAL
                     AND DTNUMERNF = P_DATARE
                     AND IDENTRSAID = P_IDENTRSAID
                     AND IDTIPOOPERNF = P_IDENTRSAID;

                  /* Retorna o ltimo sequencial do dia gerado, caso seja cancelado o RE do dia */
                  /* Retorna o maior sequencial do dia j gerado para o RE */
                  BEGIN
                     SELECT LPAD(MAX(NRNUMERNF),20,'0')
                       INTO V_ANTNUMRE
                       FROM NUMERORNF
                      WHERE CDFILIAL = P_CDFILIAL
                        AND DTNUMERNF = (SELECT MAX(DTNUMERNF)
                                           FROM NUMERORNF
                                          WHERE CDFILIAL = P_CDFILIAL
                                            AND DTNUMERNF <= (P_DATARE));
                  EXCEPTION
                       WHEN NO_DATA_FOUND THEN
                          V_ANTNUMRE := NULL;
                  END;

                  IF V_ANTNUMRE IS NULL THEN
                     V_ANTNUMRE := LPAD('0',20,'0');
                  END IF;

                  UPDATE NOVOCODIGO
                     SET NRSEQUENCIAL = V_ANTNUMRE
                   WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
               END IF;
            ELSE
               /* Verifica se j possui RE (Capa de documento) gerada para o dia */
               BEGIN
                  SELECT MAX(N.NRSEQCAPADOC)
                    INTO V_NUMERORE
                    FROM NOTAFISCAL N
                   WHERE N.CDFILIEMIT = P_CDFILIAL
                     AND N.IDENTRSAIDOP = 'S'
                     AND N.DTENTRSAID = P_DATARE
                     AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                     AND N.IDSTATUS NOT IN ('C', 'D')
                     AND NOT N.NRSEQCAPADOC IS NULL;
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     V_NUMERORE := NULL;
               END;

               IF NOT V_NUMERORE IS NULL THEN

                  BEGIN
                     SELECT MAX(N.CDPARAMEMISRE)
                       INTO V_CDREGRA
                       FROM NOTAFISCAL N
                      WHERE N.CDFILIEMIT = P_CDFILIAL
                        AND N.IDENTRSAIDOP = 'S'
                        AND N.DTENTRSAID = P_DATARE
                        AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                        AND N.IDSTATUS NOT IN ('C', 'D')
                        AND N.NRSEQCAPADOC = V_NUMERORE;
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN
                        V_CDREGRA := NULL;
                  END;

                  FOR C_NOTAFISCAL IN C_NOTAFISCALS LOOP

                     UPDATE NOTAFISCAL
                        SET NRSEQCAPADOC = V_NUMERORE,
                            CDPARAMEMISRE = V_CDREGRA
                      WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                        AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
                  END LOOP;
               END IF;

               V_REGERADO := 'S';
            END IF;

            IF V_REGERADO = 'N' THEN

               /* Verifica a data de fechamento do ms de dezembro */
               BEGIN
                  SELECT MAX(NRNUMERNF)
                    INTO V_NUMRENANO
                    FROM NUMERORNF
                   WHERE CDFILIAL = P_CDFILIAL
                     AND DTNUMERNF >= TO_DATE('01/01/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
               EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                       V_NUMRENANO := NULL;
               END;

               BEGIN
                  SELECT DTFECMES + 1
                    INTO V_DTFECMESANT
                    FROM DIAFECMES
                   WHERE CDFILIAL = P_CDFILIAL
                     AND DTFECMES BETWEEN TO_DATE('01/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1)) AND TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1));
               EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                       V_DTFECMESANT := TO_DATE('01/01/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
               END;

               BEGIN
                  SELECT MAX(NRNUMERNF)
                    INTO V_NUMRENPER
                    FROM NUMERORNF
                   WHERE CDFILIAL = P_CDFILIAL
                     AND DTNUMERNF BETWEEN V_DTFECMESANT AND P_DATARE;
               EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                       V_NUMRENPER := NULL;
               END;

               IF (V_NUMRENANO IS NULL  AND
                   V_NUMRENPER IS NULL) THEN
                  BEGIN
                     SELECT DTFECMES
                       INTO V_DTFECMES
                       FROM DIAFECMES
                      WHERE CDFILIAL = P_CDFILIAL
                        AND DTFECMES BETWEEN TO_DATE('01/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1)) AND TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1));
                  EXCEPTION
                       WHEN NO_DATA_FOUND THEN
                          V_DTFECMES := TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1));
                  END;
               ELSE
                  BEGIN
                     SELECT DTFECMES
                       INTO V_DTFECMES
                       FROM DIAFECMES
                      WHERE CDFILIAL = P_CDFILIAL
                        AND DTFECMES BETWEEN TO_DATE('01/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY')))) AND TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
                  EXCEPTION
                       WHEN NO_DATA_FOUND THEN
                          V_DTFECMES := TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
                  END;
               END IF;

               /* Verifica se existe algum RE posterior a data de fechamento do ms de dezembro do ano anterior */
               BEGIN
                  SELECT MAX(NRNUMERNF)
                    INTO V_NRMAIORRE
                    FROM NUMERORNF
                   WHERE CDFILIAL = P_CDFILIAL
                     AND DTNUMERNF >= P_DATARE
                     AND DTNUMERNF > V_DTFECMES;
               EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                       V_NRMAIORRE := NULL;
               END;


               /* Utiliza parametrizao para emisso do RE (Regras) e possui regras */
               IF (V_IDPARAMEMISRE = 'S' AND NOT V_CDPARAMEMISRE IS NULL) THEN

                  FOR C_PARAMEMIS IN C_PARAMEMISSAO LOOP

                     V_IDSEQCRIADO := 'N';

                     V_CDMODEDOCU := ' ';
                     V_CDTIPOOPER := ' ';
                     V_CDFORNECED := ' ';
                     V_CDCLIENTE  := ' ';

                     FOR C_MODEDOCU IN (SELECT CDMODDOCFISC
                                          FROM MODDOCEMISSAORE
                                         WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE) LOOP
                        V_CDMODEDOCU := V_CDMODEDOCU || C_MODEDOCU.CDMODDOCFISC || '|';
                     END LOOP;
                     FOR C_TIPOOPER IN (SELECT CDTIPOOPER
                                          FROM TIPOPERACEMISRE
                                         WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE
                                           AND IDENTRSAIDOP = P_IDENTRSAID) LOOP
                        V_CDTIPOOPER := V_CDTIPOOPER || C_TIPOOPER.CDTIPOOPER || '|';
                     END LOOP;
                     FOR C_FORNECED IN (SELECT CDFORNECED
                                          FROM FORNECEMISSAORE
                                         WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE) LOOP
                        V_CDFORNECED := V_CDFORNECED || C_FORNECED.CDFORNECED || '|';
                     END LOOP;
                     FOR C_CLIENTE IN (SELECT CDCLIENTE
                                         FROM CLIENTEMISSAORE
                                        WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE) LOOP
                        V_CDCLIENTE := V_CDCLIENTE || C_CLIENTE.CDCLIENTE || '|';
                     END LOOP;

                     FOR C_NOTAFISCAL IN C_NOTAFISCALS LOOP

                        V_IDCAPACORRES := 'S';

                        IF (NOT V_CDMODEDOCU = ' ' AND V_IDCAPACORRES = 'S') THEN
                           IF INSTR(V_CDMODEDOCU, C_NOTAFISCAL.CDMODDOCFISC) = 0 THEN
                              V_IDCAPACORRES := 'N';
                           END IF;
                        END IF;

                        IF (NOT V_CDTIPOOPER = ' ' AND V_IDCAPACORRES = 'S') THEN
                           IF INSTR(V_CDTIPOOPER, C_NOTAFISCAL.CDTIPOOPER) = 0 THEN
                              V_IDCAPACORRES := 'N';
                           END IF;
                        END IF;

                        IF (NOT V_CDFORNECED = ' ' AND V_IDCAPACORRES = 'S') OR
                           (NOT V_CDCLIENTE = ' ' AND V_IDCAPACORRES = 'S')  THEN
                           IF ((INSTR(V_CDFORNECED, C_NOTAFISCAL.CDFORNECED) = 0) AND
                               (INSTR(V_CDCLIENTE, C_NOTAFISCAL.CDCLIENTE) = 0))  THEN
                              V_IDCAPACORRES := 'N';
                           END IF;
                        END IF;

                        /* Se atende as regras de capa de documento conforme parametrizao (regras) */
                        IF V_IDCAPACORRES = 'S' THEN
                           /* Verifica se j possui RE (Capa de documento) gerada para o dia conforme regras */
                           BEGIN
                              SELECT MAX(N.NRSEQCAPADOC)
                                INTO V_NUMERORE
                                FROM NOTAFISCAL N
                               WHERE N.CDFILIEMIT = P_CDFILIAL
                                 AND N.IDENTRSAIDOP = 'S'
                                 AND N.DTENTRSAID = P_DATARE
                                 AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                                 AND N.IDSTATUS NOT IN ('C', 'D')
                                 AND NOT N.NRSEQCAPADOC IS NULL
                                 AND N.CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE;
                           EXCEPTION
                              WHEN NO_DATA_FOUND THEN
                                 V_NUMERORE := NULL;
                           END;

                           IF V_NUMERORE IS NULL THEN
                              IF V_IDSEQCRIADO <> 'S' THEN
                                 IF ((V_IDREINICIANUMRE = 'S') AND
                                     (P_DATARE > V_DTFECMES)   AND
                                     (V_NRMAIORRE IS NULL))    THEN

                                    /* Verifica se j possui RE (Capa de documento) gerada para o dia indepedendente de regras */
                                    BEGIN
                                       SELECT MAX(N.NRSEQCAPADOC)
                                         INTO V_NUMERORED
                                         FROM NOTAFISCAL N
                                        WHERE N.CDFILIEMIT = P_CDFILIAL
                                          AND N.IDENTRSAIDOP = 'S'
                                          AND N.DTENTRSAID = P_DATARE
                                          AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                                          AND N.IDSTATUS NOT IN ('C', 'D')
                                          AND NOT N.NRSEQCAPADOC IS NULL;
                                    EXCEPTION
                                       WHEN NO_DATA_FOUND THEN
                                          V_NUMERORED := NULL;
                                    END;

                                    IF V_NUMERORED IS NULL THEN
                                       UPDATE NOVOCODIGO
                                          SET NRSEQUENCIAL = LPAD('0',20,'0')
                                        WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
                                    END IF;
                                 END IF;

                                 V_PRXNUMRE := NULL;

                                 NOVO_CODIGO('RENOTAFISCAL'||P_CDFILIAL, V_PRXNUMRE, 1);
                                 V_NUMERORE := SUBSTR(V_PRXNUMRE, 11, 10);

                                 /* Insere novo nmero de RE */
                                 INSERT INTO NUMERORNF
                                   (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                                 VALUES
                                   (P_CDFILIAL, P_DATARE, P_IDENTRSAID, P_IDENTRSAID, V_NUMERORE);

                                 UPDATE PARAMFILIAL
                                    SET NRRENOTAFISC = V_NUMERORE
                                  WHERE CDFILIAL = P_CDFILIAL;

                                 V_IDSEQCRIADO := 'S';
                              END IF;
                           END IF;
                           UPDATE NOTAFISCAL
                              SET CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE,
                                  NRSEQCAPADOC = V_NUMERORE
                            WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                              AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
                        END IF;
                     END LOOP;
                  END LOOP;

                  /*   Verifica notas que no possuem nenhuma regra e gera uma capa de documento (maior capa do dia) */
                  V_IDSEQCRIADO := 'N';

                  FOR C_NOTAFISCAL IN C_NOTAFISCALS LOOP
                     IF V_IDSEQCRIADO <> 'S' THEN
                        IF ((V_IDREINICIANUMRE = 'S') AND
                            (P_DATARE > V_DTFECMES)   AND
                            (V_NRMAIORRE IS NULL))    THEN

                           /* Verifica se j possui RE (Capa de documento) gerada para o dia */
                           BEGIN
                              SELECT MAX(NRNUMERNF)
                                INTO V_NUMERORE
                                FROM NUMERORNF
                               WHERE CDFILIAL = P_CDFILIAL
                                 AND DTNUMERNF = P_DATARE
                                 AND IDENTRSAID = 'S'
                                 AND IDTIPOOPERNF = 'S';
                           EXCEPTION
                              WHEN NO_DATA_FOUND THEN
                                 V_NUMERORE := NULL;
                           END;

                           IF V_NUMERORE IS NULL THEN
                              UPDATE NOVOCODIGO
                                 SET NRSEQUENCIAL = LPAD('0',20,'0')
                               WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
                           END IF;
                        END IF;

                        V_PRXNUMRE := NULL;

                        NOVO_CODIGO('RENOTAFISCAL'||P_CDFILIAL, V_PRXNUMRE, 1);
                        V_NUMERORE := SUBSTR(V_PRXNUMRE, 11, 10);

                        /* Insere novo nmero de RE */
                        INSERT INTO NUMERORNF
                          (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                        VALUES
                          (P_CDFILIAL, P_DATARE, P_IDENTRSAID, P_IDENTRSAID, V_NUMERORE);

                        UPDATE PARAMFILIAL
                           SET NRRENOTAFISC = V_NUMERORE
                         WHERE CDFILIAL = P_CDFILIAL;

                        V_IDSEQCRIADO := 'S';
                     END IF;

                     UPDATE NOTAFISCAL
                        SET NRSEQCAPADOC = V_NUMERORE
                      WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                        AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
                  END LOOP;
               ELSE
                  /* Verifica se j possui RE (Capa de documento) gerada para o dia */
                  BEGIN
                     SELECT MAX(N.NRSEQCAPADOC)
                       INTO V_NUMERORE
                       FROM NOTAFISCAL N
                      WHERE CDFILIEMIT = P_CDFILIAL
                        AND IDENTRSAIDOP = P_IDENTRSAID
                        AND DTENTRSAID = P_DATARE
                        AND (IDSISTEMA IN ('EST','DFS','FAI') OR IDSISTEMA IS NULL)
                        AND NOT N.NRSEQCAPADOC IS NULL;
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN
                        V_NUMERORE := NULL;
                  END;

                  BEGIN
                     SELECT MAX(N.CDPARAMEMISRE)
                       INTO V_CDREGRA
                       FROM NOTAFISCAL N
                      WHERE CDFILIEMIT = P_CDFILIAL
                        AND IDENTRSAIDOP = P_IDENTRSAID
                        AND DTENTRSAID = P_DATARE
                        AND (IDSISTEMA IN ('EST','DFS','FAI') OR IDSISTEMA IS NULL)
                        AND N.NRSEQCAPADOC = V_NUMERORE;
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN
                        V_CDREGRA := NULL;
                  END;


                  V_IDSEQCRIADO := 'N';

                  FOR C_NOTAFISCAL IN C_NOTAFISCALS LOOP

                     IF V_NUMERORE IS NULL THEN
                        IF V_IDSEQCRIADO <> 'S' THEN
                           IF ((V_IDREINICIANUMRE = 'S') AND
                               (P_DATARE > V_DTFECMES)   AND
                               (V_NRMAIORRE IS NULL))    THEN
                              UPDATE NOVOCODIGO
                                 SET NRSEQUENCIAL = LPAD('0',20,'0')
                               WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
                           END IF;

                           V_PRXNUMRE := NULL;

                           NOVO_CODIGO('RENOTAFISCAL'||P_CDFILIAL, V_PRXNUMRE, 1);
                           V_NUMERORE := SUBSTR(V_PRXNUMRE, 11, 10);

                           /* Insere novo nmero de RE */
                           INSERT INTO NUMERORNF
                             (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                           VALUES
                             (P_CDFILIAL, P_DATARE, P_IDENTRSAID, P_IDENTRSAID, V_NUMERORE);

                           UPDATE PARAMFILIAL
                              SET NRRENOTAFISC = V_NUMERORE
                            WHERE CDFILIAL = P_CDFILIAL;

                           V_IDSEQCRIADO := 'S';
                        END IF;
                     END IF;

                     UPDATE NOTAFISCAL
                        SET NRSEQCAPADOC = V_NUMERORE,
                            CDPARAMEMISRE = V_CDREGRA
                      WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                        AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
                  END LOOP;
               END IF;
            END IF;
         ELSE
            P_IDERRO := 'S';
         END IF;
      END IF;
   ELSE
      V_REGERADO := 'N';

      /* Verifica se possui algum RE posterior a data do RE gerado */
      SELECT MIN(NRNUMERNF)
        INTO V_NRREPOST
        FROM NUMERORNF
       WHERE CDFILIAL = P_CDFILIAL
         AND DTNUMERNF > P_DATARE;

      IF V_NRREPOST IS NULL THEN

         /* Verifica se a data do RE  igual a data de processamento da filial,
            caso seja, ser gerado novamente o RE de notas fiscais */
         IF V_DTPROCESSA = P_DATARE THEN
           UPDATE NOTAFISCAL
              SET CDPARAMEMISRE = NULL,
                  NRSEQCAPADOC = NULL
            WHERE IDENTRSAIDOP = 'S'
              AND IDSTATUS IN ('A','E')
              AND DTAPROVACAO = P_DATARE
              AND (IDSISTEMA IN ('EST','DFS','FAI','PDV') OR IDSISTEMA IS NULL)
              AND (EXISTS (SELECT CDFILIAL
                             FROM FILIAL FL
                            WHERE NVL(CDFILIFISC,FL.CDFILIAL) = P_CDFILIAL
                              AND NOTAFISCAL.CDFILIEMIT = FL.CDFILIAL));

            DELETE FROM NUMERORNF
             WHERE CDFILIAL = P_CDFILIAL
               AND DTNUMERNF = P_DATARE
               AND IDENTRSAID = P_IDENTRSAID
               AND IDTIPOOPERNF = P_IDENTRSAID;

            /* Retorna sequencial do dia ltimo dia gerado, caso seja cancelado o RE do dia */
            BEGIN
            SELECT LPAD(MAX(NRNUMERNF),20,'0')
              INTO V_ANTNUMRE
              FROM NUMERORNF
             WHERE CDFILIAL = P_CDFILIAL
               AND DTNUMERNF = (SELECT MAX(DTNUMERNF)
                                  FROM NUMERORNF
                                 WHERE CDFILIAL = P_CDFILIAL
                                   AND DTNUMERNF <= (P_DATARE - 1));
            EXCEPTION
                 WHEN NO_DATA_FOUND THEN
                    V_ANTNUMRE := NULL;
            END;

            IF V_ANTNUMRE IS NULL THEN
               V_ANTNUMRE := LPAD('0',20,'0');
            END IF;

            UPDATE NOVOCODIGO
               SET NRSEQUENCIAL = V_ANTNUMRE
             WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
         END IF;
      ELSE
         /* Verifica se j possui RE (Capa de documento) gerada para o dia */
         BEGIN
            SELECT MAX(N.NRSEQCAPADOC)
              INTO V_NUMERORE
              FROM NOTAFISCAL N, TIPOOPERAC T
             WHERE N.IDENTRSAIDOP = 'S'
               AND IDSTATUS IN ('A','E')
               AND N.DTAPROVACAO   = P_DATARE
               AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
               AND NOT N.NRSEQCAPADOC IS NULL
               AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
               AND N.CDTIPOOPER = T.CDTIPOOPER
               AND T.IDCOMPEMISRE = 'C'
               AND (EXISTS (SELECT CDFILIAL
                              FROM FILIAL FL
                             WHERE NVL(CDFILIFISC,FL.CDFILIAL) = P_CDFILIAL
                               AND N.CDFILIEMIT = FL.CDFILIAL));
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NUMERORE := NULL;
         END;

         IF NOT V_NUMERORE IS NULL THEN

            BEGIN
               SELECT MAX(N.CDPARAMEMISRE)
                 INTO V_CDREGRA
                 FROM NOTAFISCAL N, TIPOOPERAC T
                WHERE N.IDENTRSAIDOP = 'S'
                  AND IDSTATUS IN ('A','E')
                  AND N.DTAPROVACAO   = P_DATARE
                  AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                  AND N.NRSEQCAPADOC = V_NUMERORE
                  AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                  AND N.CDTIPOOPER = T.CDTIPOOPER
                  AND T.IDCOMPEMISRE = 'C'
                  AND (EXISTS (SELECT CDFILIAL
                                 FROM FILIAL FL
                                WHERE NVL(CDFILIFISC,FL.CDFILIAL) = P_CDFILIAL
                                  AND N.CDFILIEMIT = FL.CDFILIAL));

            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_CDREGRA := NULL;
            END;

            FOR C_NOTAFISCAL IN C_NOTAFISCALSC LOOP

               UPDATE NOTAFISCAL
                  SET NRSEQCAPADOC = V_NUMERORE,
                      CDPARAMEMISRE = V_CDREGRA
                WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                  AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
            END LOOP;
         END IF;

         V_REGERADO := 'S';
      END IF;

      IF V_REGERADO = 'N' THEN

         /* Verifica a data de fechamento do ms de dezembro */
         BEGIN
            SELECT MAX(NRNUMERNF)
              INTO V_NUMRENANO
              FROM NUMERORNF
             WHERE CDFILIAL = P_CDFILIAL
               AND DTNUMERNF >= TO_DATE('01/01/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
         EXCEPTION
              WHEN NO_DATA_FOUND THEN
                 V_NUMRENANO := NULL;
         END;

         BEGIN
            SELECT DTFECMES + 1
              INTO V_DTFECMESANT
              FROM DIAFECMES
             WHERE CDFILIAL = P_CDFILIAL
               AND DTFECMES BETWEEN TO_DATE('01/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1)) AND TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1));
         EXCEPTION
              WHEN NO_DATA_FOUND THEN
                 V_DTFECMESANT := TO_DATE('01/01/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
         END;

         BEGIN
            SELECT MAX(NRNUMERNF)
              INTO V_NUMRENPER
              FROM NUMERORNF
             WHERE CDFILIAL = P_CDFILIAL
               AND DTNUMERNF BETWEEN V_DTFECMESANT AND P_DATARE;
         EXCEPTION
              WHEN NO_DATA_FOUND THEN
                 V_NUMRENPER := NULL;
         END;

         IF (V_NUMRENANO IS NULL  AND
             V_NUMRENPER IS NULL) THEN

            BEGIN
               SELECT DTFECMES
                 INTO V_DTFECMES
                 FROM DIAFECMES
                WHERE CDFILIAL = P_CDFILIAL
                  AND DTFECMES BETWEEN TO_DATE('01/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1)) AND TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1));
            EXCEPTION
                 WHEN NO_DATA_FOUND THEN
                    V_DTFECMES := TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))-1));
            END;

         ELSE
            BEGIN
               SELECT DTFECMES
                 INTO V_DTFECMES
                 FROM DIAFECMES
                WHERE CDFILIAL = P_CDFILIAL
                  AND DTFECMES BETWEEN TO_DATE('01/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY')))) AND TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
            EXCEPTION
                 WHEN NO_DATA_FOUND THEN
                    V_DTFECMES := TO_DATE('31/12/'|| TO_CHAR(TO_NUMBER(TO_CHAR(P_DATARE,'YYYY'))));
            END;
         END IF;

         /* Verifica se existe algum RE posterior a data de fechamento do ms de dezembro do ano anterior */
         BEGIN
            SELECT MAX(NRNUMERNF)
              INTO V_NRMAIORRE
              FROM NUMERORNF
             WHERE CDFILIAL = P_CDFILIAL
               AND DTNUMERNF >= P_DATARE
               AND DTNUMERNF > V_DTFECMES;
         EXCEPTION
              WHEN NO_DATA_FOUND THEN
                 V_NRMAIORRE := NULL;
         END;

         /* Utiliza parametrizao para emisso do RE (Regras) e possui regras */
         IF (V_IDPARAMEMISRE = 'S' AND NOT V_CDPARAMEMISRE IS NULL) THEN

            FOR C_PARAMEMIS IN C_PARAMEMISSAO LOOP

               V_IDSEQCRIADO := 'N';

               V_CDMODEDOCU := ' ';
               V_CDTIPOOPER := ' ';
               V_CDFORNECED := ' ';
               V_CDCLIENTE  := ' ';

               FOR C_MODEDOCU IN (SELECT CDMODDOCFISC
                                    FROM MODDOCEMISSAORE
                                   WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE) LOOP
                  V_CDMODEDOCU := V_CDMODEDOCU || C_MODEDOCU.CDMODDOCFISC || '|';
               END LOOP;
               FOR C_TIPOOPER IN (SELECT CDTIPOOPER
                                    FROM TIPOPERACEMISRE
                                   WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE
                                     AND IDENTRSAIDOP = P_IDENTRSAID) LOOP
                  V_CDTIPOOPER := V_CDTIPOOPER || C_TIPOOPER.CDTIPOOPER || '|';
               END LOOP;
               FOR C_FORNECED IN (SELECT CDFORNECED
                                    FROM FORNECEMISSAORE
                                   WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE) LOOP
                  V_CDFORNECED := V_CDFORNECED || C_FORNECED.CDFORNECED || '|';
               END LOOP;
               FOR C_CLIENTE IN (SELECT CDCLIENTE
                                   FROM CLIENTEMISSAORE
                                  WHERE CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE) LOOP
                  V_CDCLIENTE := V_CDCLIENTE || C_CLIENTE.CDCLIENTE || '|';
               END LOOP;

               FOR C_NOTAFISCAL IN C_NOTAFISCALSC LOOP

                  V_IDCAPACORRES := 'S';

                  IF (NOT V_CDMODEDOCU = ' ' AND V_IDCAPACORRES = 'S') THEN
                     IF INSTR(V_CDMODEDOCU, C_NOTAFISCAL.CDMODDOCFISC) = 0 THEN
                        V_IDCAPACORRES := 'N';
                     END IF;
                  END IF;

                  IF (NOT V_CDTIPOOPER = ' ' AND V_IDCAPACORRES = 'S') THEN
                     IF INSTR(V_CDTIPOOPER, C_NOTAFISCAL.CDTIPOOPER) = 0 THEN
                        V_IDCAPACORRES := 'N';
                     END IF;
                  END IF;

                  IF (NOT V_CDFORNECED = ' ' AND V_IDCAPACORRES = 'S') OR
                     (NOT V_CDCLIENTE = ' ' AND V_IDCAPACORRES = 'S')  THEN
                     IF ((INSTR(V_CDFORNECED, C_NOTAFISCAL.CDFORNECED) = 0) AND
                         (INSTR(V_CDCLIENTE, C_NOTAFISCAL.CDCLIENTE) = 0))  THEN
                        V_IDCAPACORRES := 'N';
                     END IF;
                  END IF;

                  /* Se atende as regras de capa de documento conforme parametrizao (regras) */
                  IF V_IDCAPACORRES = 'S' THEN
                     /* Verifica se j possui RE (Capa de documento) gerada para o dia (RE CENTRALIZADORA) */
                     BEGIN
                        SELECT MAX(N.NRSEQCAPADOC)
                          INTO V_NUMERORE
                          FROM NOTAFISCAL N, TIPOOPERAC T
                         WHERE N.IDENTRSAIDOP = 'S'
                           AND IDSTATUS IN ('A','E')
                           AND N.DTAPROVACAO   = P_DATARE
                           AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                           AND NOT N.NRSEQCAPADOC IS NULL
                           AND N.CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE
                           AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                           AND N.CDTIPOOPER = T.CDTIPOOPER
                           AND T.IDCOMPEMISRE = 'C'
                           AND (EXISTS (SELECT CDFILIAL
                                          FROM FILIAL FL
                                         WHERE NVL(CDFILIFISC,FL.CDFILIAL) = P_CDFILIAL
                                           AND N.CDFILIEMIT = FL.CDFILIAL));

                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_NUMERORE := NULL;
                     END;

                     IF V_NUMERORE IS NULL THEN
                        IF V_IDSEQCRIADO <> 'S' THEN
                           IF ((V_IDREINICIANUMRE = 'S') AND
                               (P_DATARE > V_DTFECMES)   AND
                               (V_NRMAIORRE IS NULL))    THEN

                              /* Verifica se j possui RE (Capa de documento) gerada para o dia (RE CENTRALIZADORA) indepedente de regras */
                              BEGIN
                                 SELECT MAX(N.NRSEQCAPADOC)
                                   INTO V_NUMERORED
                                   FROM NOTAFISCAL N, TIPOOPERAC T
                                  WHERE N.IDENTRSAIDOP = 'S'
                                    AND IDSTATUS IN ('A','E')
                                    AND N.DTAPROVACAO   = P_DATARE
                                    AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                                    AND NOT N.NRSEQCAPADOC IS NULL
                                    AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                                    AND N.CDTIPOOPER = T.CDTIPOOPER
                                    AND T.IDCOMPEMISRE = 'C'
                                    AND (EXISTS (SELECT CDFILIAL
                                                   FROM FILIAL FL
                                                  WHERE NVL(CDFILIFISC,FL.CDFILIAL) = P_CDFILIAL
                                                    AND N.CDFILIEMIT = FL.CDFILIAL));

                              EXCEPTION
                                 WHEN NO_DATA_FOUND THEN
                                    V_NUMERORED := NULL;
                              END;

                              IF V_NUMERORED IS NULL THEN
                                 UPDATE NOVOCODIGO
                                    SET NRSEQUENCIAL = LPAD('0',20,'0')
                                  WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
                              END IF;
                           END IF;

                           V_PRXNUMRE := NULL;

                           NOVO_CODIGO('RENOTAFISCAL'||P_CDFILIAL, V_PRXNUMRE, 1);
                           V_NUMERORE := SUBSTR(V_PRXNUMRE, 11, 10);

                           /* Insere novo nmero de RE */
                           INSERT INTO NUMERORNF
                             (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                           VALUES
                             (P_CDFILIAL, P_DATARE, P_IDENTRSAID, P_IDENTRSAID, V_NUMERORE);

                           UPDATE PARAMFILIAL
                              SET NRRENOTAFISC = V_NUMERORE
                            WHERE CDFILIAL = P_CDFILIAL;

                           V_IDSEQCRIADO := 'S';
                        END IF;
                     END IF;
                     UPDATE NOTAFISCAL
                        SET CDPARAMEMISRE = C_PARAMEMIS.CDPARAMEMISRE,
                            NRSEQCAPADOC = V_NUMERORE
                      WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                        AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
                  END IF;
               END LOOP;
            END LOOP;
            /*   Verifica notas que no possuem nenhuma regra e gera uma capa de documento (maior capa do dia) */
            V_IDSEQCRIADO := 'N';

            FOR C_NOTAFISCAL IN C_NOTAFISCALSC LOOP
               IF V_IDSEQCRIADO <> 'S' THEN
                  IF ((V_IDREINICIANUMRE = 'S') AND
                      (P_DATARE > V_DTFECMES)   AND
                      (V_NRMAIORRE IS NULL))    THEN

                     /* Verifica se j possui RE (Capa de documento) gerada para o dia */
                     BEGIN
                     SELECT MAX(NRNUMERNF)
                       INTO V_NUMERORE
                       FROM NUMERORNF
                      WHERE CDFILIAL = P_CDFILIAL
                        AND DTNUMERNF = P_DATARE
                        AND IDENTRSAID = 'S'
                        AND IDTIPOOPERNF = 'S';
                      EXCEPTION
                         WHEN NO_DATA_FOUND THEN
                            V_NUMERORE := NULL;
                         END;

                     IF V_NUMERORE IS NULL THEN
                        UPDATE NOVOCODIGO
                           SET NRSEQUENCIAL = LPAD('0',20,'0')
                         WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
                     END IF;
                  END IF;

                  V_PRXNUMRE := NULL;

                  NOVO_CODIGO('RENOTAFISCAL'||P_CDFILIAL, V_PRXNUMRE, 1);
                  V_NUMERORE := SUBSTR(V_PRXNUMRE, 11, 10);

                  /* Insere novo nmero de RE */
                  INSERT INTO NUMERORNF
                    (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                  VALUES
                    (P_CDFILIAL, P_DATARE, P_IDENTRSAID, P_IDENTRSAID, V_NUMERORE);

                  UPDATE PARAMFILIAL
                     SET NRRENOTAFISC = V_NUMERORE
                   WHERE CDFILIAL = P_CDFILIAL;

                  V_IDSEQCRIADO := 'S';
               END IF;

               UPDATE NOTAFISCAL
                  SET NRSEQCAPADOC = V_NUMERORE
                WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                  AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
            END LOOP;
         ELSE
            /* Verifica se j possui RE (Capa de documento) gerada para o dia (RE CENTRALIZADORA) */
            BEGIN
               SELECT MAX(N.NRSEQCAPADOC)
                 INTO V_NUMERORE
                 FROM NOTAFISCAL N, TIPOOPERAC T
                WHERE N.IDENTRSAIDOP = 'S'
                  AND IDSTATUS IN ('A','E')
                  AND N.DTAPROVACAO   = P_DATARE
                  AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                  AND NOT N.NRSEQCAPADOC IS NULL
                  AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                  AND N.CDTIPOOPER = T.CDTIPOOPER
                  AND T.IDCOMPEMISRE = 'C'
                  AND (EXISTS (SELECT CDFILIAL
                                 FROM FILIAL FL
                                WHERE NVL(CDFILIFISC,FL.CDFILIAL) = P_CDFILIAL
                                  AND N.CDFILIEMIT = FL.CDFILIAL));

            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_NUMERORE := NULL;
            END;

            V_IDSEQCRIADO := 'N';

            FOR C_NOTAFISCAL IN C_NOTAFISCALSC LOOP
               IF V_NUMERORE IS NULL THEN
                  IF V_IDSEQCRIADO <> 'S' THEN
                     IF ((V_IDREINICIANUMRE = 'S') AND
                         (P_DATARE > V_DTFECMES)   AND
                         (V_NRMAIORRE IS NULL))    THEN
                        UPDATE NOVOCODIGO
                           SET NRSEQUENCIAL = LPAD('0',20,'0')
                         WHERE CDCONTADOR = 'RENOTAFISCAL' || P_CDFILIAL;
                     END IF;

                     V_PRXNUMRE := NULL;

                     NOVO_CODIGO('RENOTAFISCAL'||P_CDFILIAL, V_PRXNUMRE, 1);
                     V_NUMERORE := SUBSTR(V_PRXNUMRE, 11, 10);

                     /* Insere novo nmero de RE */
                     INSERT INTO NUMERORNF
                       (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                     VALUES
                       (P_CDFILIAL, P_DATARE, P_IDENTRSAID, P_IDENTRSAID, V_NUMERORE);

                     UPDATE PARAMFILIAL
                        SET NRRENOTAFISC = V_NUMERORE
                      WHERE CDFILIAL = P_CDFILIAL;

                     V_IDSEQCRIADO := 'S';
                  END IF;
               END IF;

               UPDATE NOTAFISCAL
                  SET NRSEQCAPADOC = V_NUMERORE
                WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                  AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
            END LOOP;
         END IF;
      END IF;
   END IF;
   COMMIT;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('GERA_RE','6.22.XXX'); END;
/
